==> online_judge_tools-10.0.3-py3-none-any.whl: onlinejudge_command/download_history.py <==
# Python Version: 3.x
import json
import pathlib
import time
import traceback
from typing import *

import onlinejudge_command.logging as log
import onlinejudge_command.utils as utils
from onlinejudge_workaround_for_conflict.type import Problem


class DownloadHistory(object):
    def __init__(self, path: pathlib.Path = utils.user_cache_dir / 'download-history.jsonl'):
        self.path = path

    def add(self, problem: Problem, *, directory: pathlib.Path) -> None:
        log.status('append the downloading history: %s', self.path)
        self.path.parent.mkdir(parents=True, exist_ok=True)
        with open(str(self.path), 'a') as fh:
            fh.write(json.dumps({
                'timestamp': int(time.time()),  # this should not be int, but Python's strptime is too weak and datetime.fromisoformat is from 3.7
                'directory': str(directory),
                'url': problem.get_url(),
            }) + '\n')
        self._flush()

    def remove(self, *, directory: pathlib.Path) -> None:
        if not self.path.exists():
            return
        log.status('clear the downloading history for this directory: %s', self.path)
        with open(str(self.path)) as fh:
            history_lines = fh.readlines()
        with open(str(self.path), 'w') as fh:
            pred = lambda line: pathlib.Path(json.loads(line)['directory']) != directory
            fh.write(''.join(filter(pred, history_lines)))

    def _flush(self) -> None:
        # halve the size if it is more than 1MiB
        if self.path.stat().st_size >= 1024 * 1024:
            with open(str(self.path)) as fh:
                history_lines = fh.readlines()
            with open(str(self.path), 'w') as fh:
                fh.write(''.join(history_lines[:-len(history_lines) // 2]))
            log.status('halve history at: %s', self.path)

    def get(self, *, directory: pathlib.Path) -> List[str]:
        if not self.path.exists():
            return []

        log.status('read history from: %s', self.path)
        found = set()
        with open(str(self.path)) as fh:
            for line in fh:
                try:
                    data = json.loads(line)
                except json.decoder.JSONDecodeError as e:
                    log.warning('corrupted line found in: %s', self.path)
                    log.debug('%s', traceback.format_exc())
                    continue
                if pathlib.Path(data['directory']) == directory:
                    found.add(data['url'])
        log.status('found urls in history:\n%s', '\n'.join(found))
        return list(found)

==> online_judge_tools-10.0.5-py3-none-any.whl: onlinejudge_command/download_history.py <==
# Python Version: 3.x
import json
import pathlib
import time
import traceback
from typing import *

import onlinejudge_command.logging as log
import onlinejudge_command.utils as utils

from onlinejudge.type import Problem


class DownloadHistory(object):
    def __init__(self, path: pathlib.Path = utils.user_cache_dir / 'download-history.jsonl'):
        self.path = path

    def add(self, problem: Problem, *, directory: pathlib.Path) -> None:
        log.status('append the downloading history: %s', self.path)
        self.path.parent.mkdir(parents=True, exist_ok=True)
        with open(str(self.path), 'a') as fh:
            fh.write(json.dumps({
                'timestamp': int(time.time()),  # this should not be int, but Python's strptime is too weak and datetime.fromisoformat is from 3.7
                'directory': str(directory),
                'url': problem.get_url(),
            }) + '\n')
        self._flush()

    def remove(self, *, directory: pathlib.Path) -> None:
        if not self.path.exists():
            return
        log.status('clear the downloading history for this directory: %s', self.path)
        with open(str(self.path)) as fh:
            history_lines = fh.readlines()
        with open(str(self.path), 'w') as fh:
            pred = lambda line: pathlib.Path(json.loads(line)['directory']) != directory
            fh.write(''.join(filter(pred, history_lines)))

    def _flush(self) -> None:
        # halve the size if it is more than 1MiB
        if self.path.stat().st_size >= 1024 * 1024:
            with open(str(self.path)) as fh:
                history_lines = fh.readlines()
            with open(str(self.path), 'w') as fh:
                fh.write(''.join(history_lines[:-len(history_lines) // 2]))
            log.status('halve history at: %s', self.path)

    def get(self, *, directory: pathlib.Path) -> List[str]:
        if not self.path.exists():
            return []

        log.status('read history from: %s', self.path)
        found = set()
        with open(str(self.path)) as fh:
            for line in fh:
                try:
                    data = json.loads(line)
                except json.decoder.JSONDecodeError as e:
                    log.warning('corrupted line found in: %s', self.path)
                    log.debug('%s', traceback.format_exc())
                    continue
                if pathlib.Path(data['directory']) == directory:
                    found.add(data['url'])
        log.status('found urls in history:\n%s', '\n'.join(found))
        return list(found)
